#!/usr/bin/env python
# -*- coding:utf-8 -*-
from numpy import *
from sklearn.svm import SVC
from sklearn.model_selection import StratifiedKFold
from sklearn.metrics import roc_curve, auc, roc_auc_score, pairwise_kernels

import matplotlib.pyplot as plt

# Import multiple kernel learning method
from MKLpy.algorithms import EasyMKL
from MKLpy.model_selection import cv3

fs = loadtxt('data/seq.dat', dtype=int, delimiter=',')
fp = loadtxt('data/ppi.dat', dtype=int, delimiter=',')
lb = loadtxt('data/labels.dat', dtype=int, delimiter=',')
nm = loadtxt('data/names.dat', dtype=str, delimiter=',')

gammas = arange(1, 6, 1)
ks_list, kp_list = [], []
for gamma in gammas:
    ks_list.append(pairwise_kernels(fs, metric='rbf', gamma=gamma * 1. / fs.shape[1]))
    kp_list.append(pairwise_kernels(fp, metric='rbf', gamma=gamma * 1. / fp.shape[1]))

kernel = ks_list + kp_list

# Cross validation
y_all = zeros_like(lb, dtype=float)
for i, ks in enumerate(nm):
    y_true = lb[:, i]
    skf = StratifiedKFold(n_splits=10)
    kernel = array(kernel)
    y_pred = zeros(len(y_true), dtype=float)
    for train_index, test_index in skf.split(zeros(len(y_true)), y_true):
        tr_index, ts_index = cv3(train_index, test_index, kernel.shape[0])
        K_train, K_test = kernel[tr_index], kernel[ts_index]
        y_train, y_test = y_true[train_index], y_true[test_index]
        clf = EasyMKL(lam=0.01, kernel='precomputed', base=SVC(class_weight='balanced')).fit(K_train, y_train)
        y_pred[test_index] = clf.decision_function(K_test)
    val = roc_auc_score(y_true, y_pred)
    y_all[:, i] = y_pred
    print '%s: %.1f%%' % (nm[i], val * 100)

# Draw the roc curve
m = len(y_all[0])
ii = int(round(sqrt(m)))
fig, axes = plt.subplots(ii, ii, figsize=(15, 12))
for i, ax in zip(xrange(m), axes.flat):
    fpr, tpr, _ = roc_curve(lb[:, i], y_all[:, i])
    val = auc(fpr, tpr)
    ax.plot(fpr, tpr, label='val = %.1f%%' % (val * 100))
    ax.set_xlabel('false positive rate')
    ax.set_ylabel('true positive rate')
    ax.set_title(nm[i ])
    ax.legend(loc='lower right')
fig.tight_layout()

# Save the result
plt.savefig('ksrMKL.png', dpi=300)
savetxt('ksrMKL.dat', y_all, fmt='%.4f', delimiter=',')
print 'over!'
